# -*- coding: utf8 -*-

from django.db import models
from portailuser.models import Ecole, PortailUser
from ldap_inp import LdapEleve, LdapEleveAvecPhoto, LdapGroupe, L
from . import search_tools

import re


def formater_telephone(val):
    val = re.sub("[^0-9+]", "", val)

    if val[0:2] == "00":
        val = "+" + val[2:]

    if val[0] == "0" and len(val) == 10:
        val = "+33" + val[1:]

    return val


def chercher_staff_groupe(nom_groupe, ecole, klass):
    if not set("()*").isdisjoint(nom_groupe):
        return []

    try:
        if ecole:
            groupe = LdapGroupe.objects.get(cn=nom_groupe + "-" + ecole.ident)
        else:
            groupe = LdapGroupe.objects.get(cn=nom_groupe + "-" + Ecole.inp.ident)
    except LdapGroupe.DoesNotExist:
        try:
            if ecole:
                groupe = LdapGroupe.objects.get(displayName=nom_groupe, ecole=ecole)
            else:
                groupe = LdapGroupe.objects.get(displayName=nom_groupe)
        except LdapGroupe.DoesNotExist:
            try:
                groupe = LdapGroupe.objects.get(displayName=nom_groupe)
            except LdapGroupe.DoesNotExist:
                return []

    filtre = L(uid=groupe.memberUid)

    return klass.objects.filter(filtre)


def requete(texte_rech, ecole_rech=None, avec_photo=False):
    # Champs sur lesquels on recherche
    champs = (
        "snSearch",
        "givenNameSearch",
        "nickname",
        "promo",
        "filiere",
        "genre",
        "mailAnnexe",
    )

    if len(texte_rech) < 2:
        return []

    klass = LdapEleveAvecPhoto if avec_photo else LdapEleve

    resultats = []
    if re.match("^[0-9 .+-]{5,20}$", texte_rech):
        tel = formater_telephone(texte_rech)
        filtre = L(homePhone=tel) | L(mobile=tel)
        if ecole_rech and ecole_rech != Ecole.inp:
            filtre = L(ecole=ecole_rech) & filtre

        resultats = klass.objects.filter(filtre)

        if len(resultats) == 1:
            return resultats[0]

    if len(resultats) == 0:
        symboles = texte_rech.split()
        combinaisons = search_tools.combispace(symboles)
        if combinaisons is None:
            return []
        for combi in combinaisons:
            filtre = search_tools.gen_filterldap(combi, champs)
            if filtre is not None:
                if ecole_rech and ecole_rech != Ecole.inp:
                    filtre = L(ecole=ecole_rech) & filtre
                resultats = klass.objects.filter(filtre)
                if len(resultats) != 0:
                    break

        if len(resultats) == 0 or symboles[0].lower() == "club" or symboles[0].lower() == "groupe":
            if symboles[0].lower() == "club" or symboles[0].lower() == "groupe":
                sym = symboles[-1]
            else:
                sym = texte_rech
            resultats = chercher_staff_groupe(sym, ecole_rech, klass)

    if len(resultats) == 1:
        return resultats[0]

    # tri des resultats par nom de famille
    resultats = sorted(resultats, key=lambda resultat: resultat.nom)

    return resultats


class Consultation_Annuaire(models.Model):
    date = models.DateField(auto_now_add=True)
    nb_acces = models.IntegerField()

    class Meta:
        abstract = True


class Consultation_Annuaire_PortailUser(Consultation_Annuaire):
    user = models.ForeignKey(PortailUser, on_delete=models.CASCADE)


class Consultation_Annuaire_IP(Consultation_Annuaire):
    ip = models.CharField(max_length=16)
